class EventManagerException(Exception):
    pass


class CannotProcessSyncEvent(EventManagerException):
    pass


class AsyncTargetForEventWithNoData(EventManagerException):
    pass


class AsyncTargetForEventWithNoUtf8Data(EventManagerException):
    pass


class AsyncTargetForEventWithNoJsonData(EventManagerException):
    pass


class AsyncTargetForEventWithDataWithoutId(EventManagerException):
    pass


class CannotProcessATargetAndAsyncTarget(EventManagerException):
    pass
