from messagebus import TokenCreator, TokenPriority


MSGGRP_SLCTRL_ORDERMGR = "A"

TK_SLCTRL_OMGR_DOSALE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="1",
)

TK_SLCTRL_OMGR_ORDERPICT = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="2"
)

TK_SLCTRL_OMGR_CREATEORDER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="18"
)

TK_SLCTRL_OMGR_LISTORDERS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="D"
)

TK_SLCTRL_OMGR_VOIDLINE = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="3",
)

TK_SLCTRL_OMGR_DOOPTION = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="A",
)

TK_SLCTRL_OMGR_CHANGEMODIFIER = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=MSGGRP_SLCTRL_ORDERMGR,
    number="13",
)


def get_tokens():
    return (
        TK_SLCTRL_OMGR_CHANGEMODIFIER,
        TK_SLCTRL_OMGR_DOOPTION,
        TK_SLCTRL_OMGR_DOSALE,
        TK_SLCTRL_OMGR_ORDERPICT,
        TK_SLCTRL_OMGR_CREATEORDER,
        TK_SLCTRL_OMGR_LISTORDERS,
        TK_SLCTRL_OMGR_VOIDLINE,
    )
