from functions_authorization import get_function_authorization_level_from_config
from get_keyboard_comments_config import get_keyboard_comments_config
from get_min_value_to_ask_bordereau_justify_config import get_min_value_to_ask_bordereau_justify_config
from show_numpad_for_customer_name import show_numpad_for_customer_name
from get_production_config import get_hold_time_config
from should_ask_pager_number import should_ask_pager_number
from should_ask_delivery_type import should_ask_delivery_type
from get_default_delivery_type import get_default_delivery_type
from get_pre_account_allow_printing import get_pre_account_allow_printing
from get_hide_unpriced_items_from_total_report import get_hide_unpriced_items_from_total_report
from get_pre_account_with_discount import get_pre_account_with_discount
from can_cancel_tef_payment_on_error import can_cancel_tef_payment_on_error
from ._DeliveryCoupon import must_print_delivery_coupon
from should_ask_loyalty_message import (
    should_ask_loyalty_message_on_total,
    should_ask_loyalty_on_payment,
)
from can_create_table_parallel_order import can_create_table_parallel_order
from get_close_operator_config import get_close_operator_conference_type
from considers_tef_on_bordereau import considers_tef_on_bordereau
