class ProductServiceException(Exception):
    pass


class InvalidPartCode(Exception):
    def __init__(
        self,           # type: InvalidPartCode
        part_code,      # type: int
    ):
        # type: (...) -> None

        self.part_code = part_code
