from application.model.configuration import DefaultManagerConfig


class BinaryConfigurations(DefaultManagerConfig):
    def __init__(self):
        super(BinaryConfigurations, self).__init__()

        self.endpoints = Endpoints()
        self.force_update_days = 5
        self.restrict_days_update = []
        self.chrome_url = ""
        self.chrome_version_url = ""
        self.chrome_update_enabled = False


class Endpoints(object):
    def __init__(self):
        self.download_loaders = None
