from sysactions import get_model


def get_pos_language(
    pos_id,  # type: str
):
    # type: (...) -> str

    try:
        model = get_model(posid=pos_id)
        language = model.find("Language").get("name").lower()
    except Exception as _:
        language = "pt_br"

    return language
