from enum import Enum


class PublicationAction(Enum):
    SUCCESSFULLY_PUBLISHED = "SUCCESSFULLY_PUBLISHED"
    UNSUCCESSFULLY_PUBLISHED = "UNSUCCESSFULLY_PUBLISHED"
    ASYNC_PUBLISHED = "ASYNC_PUBLISHED"
    ASYNC_REPLY_RECEIVED = "ASYNC_REPLY_RECEIVED"
