from typing import List


class PickupAreaSector(object):
    def __init__(
        self,               # type: PickupAreaSector
        name,               # type: str
        excluded_pos,       # type: List[int]
    ):
        # type: (...) -> None

        self.name = name
        self.excluded_pos = excluded_pos
