from datetime import datetime


class UpdateData(object):
    def __init__(
        self,                           # type: UpdateData
        update_id=None,                 # type: int
        update_name=None,               # type: str
        obtained_date=None,             # type: datetime
        downloaded_date=None,           # type: datetime
        backup_date=None,               # type: datetime
        applied_date=None,              # type: datetime
        notified_date=None,             # type: datetime
        server_update_identifier=None,  # type: str
    ):
        # type: (...) -> None

        self.update_id = update_id
        self.update_name = update_name
        self.obtained_date = obtained_date
        self.downloaded_date = downloaded_date
        self.backup_date = backup_date
        self.applied_date = applied_date
        self.notified_date = notified_date
        self.server_update_identifier = server_update_identifier
