from setuptools import (
    setup,
    find_packages,
)


setup(
    name="TableManagerApi",
    version="1.2.0",
    packages=find_packages(where="."),
    install_requires=["MessageBus==1.1.0"],
    dependency_links=["../salecomp/api"],
    url="https://www.e-deploy.com.br",
    author="Claudinei Nascimento",
    author_email="claudinei.nascimento@e-deploy.com.br"
)
