# -*- coding: utf-8 -*-

from datetime import datetime


class PosUser(object):
    def __init__(self, user_id, user_name, long_name, password, level, status, admission_date, termination_date):
        # type: (int, str, str, str, int, bool, datetime, datetime) -> None

        self.user_id = user_id
        self.user_name = user_name
        self.long_name = long_name
        self.password = password
        self.level = level
        self.status = status
        self.admission_date = admission_date
        self.termination_date = termination_date
